package program;

import java.awt.event.KeyEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Contains constants and standard settings.
 * 
 * @since wiki2xhtml 3.3
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public final class Constants {

	
	/** Directories */
	public static final class Directories {
		public static final String style = "style";
		public static final String userHome = java.lang.System.getProperty("user.home");
		public static final String workingDir = java.lang.System.getProperty("user.dir");
		public static final String workingDir2 = java.lang.System.getProperty("user.dir") + File.separatorChar;
		public static final String settingsDir = userHome + File.separatorChar + ".wiki2xhtml";
		public static final String imagePages = "imagePages";
		public static final String target = "html";
	}
	
	/** Standard files, like the configuration file */
	public static final class Files {
		public static final File settings = new File(Constants.Directories.settingsDir + File.separatorChar + "settings.txt");
		public static final String reck = "reck.html";
		public static final String imageTemplate = "image-template.html";
		public static final String cssSettings = "css-settings.txt";
	}
	
	/** wiki2xhtml settings, like the version number */
	public static final class Wiki2xhtml {
		
		public static final String progName = "wiki2xhtml";
		public static final String versionNumber = "3.3.2";
		public static final String versionDate = "Dec 17, 2008";
		/** Old versions which will be ignored by the update checker. */
		public static final String[] versionsTillNow = new String[] { "3.3.1", "3.3", "3.2.1", "3.1.0", "3.0.4" };
		public static final String version = versionNumber + " (" + versionDate + ")";
		public static final String webpage = "http://wiki2xhtml.sf.net";
		public static final String forum = "http://forum.granjow.net";
		public static final URL[] getUpdateURLs() {
			try {
				return new URL[] { new URL("http://granjow.net/wiki2xhtml") };
			} catch (MalformedURLException e) {
				e.printStackTrace();
				return null;
			}
		}
		
	}
	
	/** Default settings like thumbnail width */
	public static final class Standards {
		
		public static final Locale locale = Locale.ENGLISH;
		
		public static String widthImgImagepages = "700px";
		public static String widthThumbs = "200px";
		public static String galleryWidthThumbs = "125px";
		
		public static byte galleryImagesPerLine = 3;
		
		public static final boolean useConsistentGallery = true;
		public static final boolean useImageDescAsCaption = true;
		public static final boolean useImagenameAsCaption = false;
	}

	
	/** Command line arguments for wiki2xhtml */
	public static final class Arguments {
		
		public static final Args stdArgs = new Args("-m menu.txt -f footer.txt -c common.txt ");
		
		public static final Args stdArgsHelpDe = new Args("-m menu-de.txt -f footer-de.txt -c common-de.txt " +
		"-s hd --td help err404.php help-de.txt doc-de.txt index-de.txt download-de.txt quickstart-de.txt " +
		"news-de.txt changelog-de.txt faq-de.txt examples-de.txt ex-php-reload-de.php " +
		"php-de.txt doc-design-de.txt quickref-de.txt about-de.txt links-de.txt usage-de.txt ");
		
		public static final Args stdArgsHelpEn = new Args(stdArgs.getArgs() +
		" -s hd --td help err404.php index.txt help.txt doc.txt download.txt quickstart.txt quickstart-ru.txt quickstart-it.txt news.txt faq.txt php.txt " +
		"examples.txt ex-php-reload.php doc-design.txt quickref.txt about.txt changelog.txt testpage.txt links.txt usage.txt ");
		
		/** Special command line arguments */
		public static final class Special {
			
			public static final String menufileUpdater = "menu";
			public static final String templateUpdater = "template";
			
		}
		
		/** Command line arguments for wiki2xhtml */
		public static final class Arg {
			
			public static final String menu = "-m";
			public static final String style = "-s";
			public static final String common = "-c";
			public static final String targetDir = "--td";
			public static final String footer = "-f";
			
		}
		
		/** Combined command line arguments for wiki2xhtml */
		public static final class CombinedArg {
			
			public static final String common = "--common=";
			public static final String footer = "--footer=";
			public static final String lang = "--lang=";
			public static final String menu = "--menu-file=";
			public static final String style = "--style=";
			public static final String targetDir = "--target-dir=";
		}
		
		/** Command line flags for wiki2xhtml with no further arguments */
		public static final class Flags {

			public static final String dead = "--dead";
			public static final String debug = "--debug";
			public static final String help = "-h";
			public static final String helpLong = "--help";
			public static final String helpfilesDe = "--helpfiles-de";
			public static final String helpfiles = "--helpfiles";
			public static final String noupdatecheck = "--noupdatecheck";
			public static final String standard = "--standard";
			public static final String silent = "--silent";
			public static final String verbose = "--verbose";
			public static final String updateCheck = "--www";
			
			public static final String onlyCode = "--only-code";
			public static final String stdout = "--stdout";
			public static final String removeLinebreaks = "--remove-linebreaks";
			public static final String typos = "--typos";
			
			public static final String gui = "--gui";
			
		}
		
		public static final String[] obsoleteFlags = new String[] {"-pn", "-pns", "--header" };
		public static final String[] obsoleteArgs = new String[] {"-i", "-t", "--index-file", "--title="};
		public static final String[] obsoleteCombined = new String[] {"--header=",  };
		
	}
	
	/** Arguments for images */
	public static final class Images {
		
		/** Direct link, no image page */
		public static final String direct = "direct";
		/** Image is small, do not enlarge it on the image page */
		public static final String small = "small";
		/** Insert a thumbnail with border and image page*/
		public static final String thumb = "thumb";
		/** Self-defined path to the thumbnail */
		public static final String pathThumb = "thumb=";
		
		/** Image caption */
		public static final String captionShort = "c=";
		/** Image caption */
		public static final String caption = "caption=";
		/** Long description of the image */
		public static final String longDesc = "ld=";
		
	}
	
	/** Menu tags which may followed after a <code>#!</code> at line beginning in the menu file. */
	public static final class MenuTags {
		
		public static final String linkDeactivate = "link_deactivate";
		public static final String linkDeactivateNot = "link_deactivate0";
		
		public static final String linkStrong = "link_strong";
		public static final String linkStrongNot = "link_strong0";
		
	}
	
	/** Positions */
	public static final class Position {
		
		public static final String left = "left";
		public static final String center = "center";
		public static final String right = "right";
		
	}
	
	/** Tags used in templates */
	public static final class TemplateTags {
		
		public static final String caption = "[caption]";
		/** Direct link to image */
		public static final String imagePath = "[imagepath]";
		public static final String imageName = "[imagename]";
		public static final String imageCaption = "[imagecaption]";
		public static final String pre = "[pre]";
		public static final String back = "[back]";
		public static final String nextImage = "[nextimage]";
		public static final String prevImage = "[previmage]";
		public static final String width = "[width]";
		
		public static final String pos = "[pos]";
		/** Link to image or image page */
		public static final String link = "[link]";
		/** Thumbnail source */
		public static final String thumb = "[thumb]";
		public static final String desc = "[desc]";
		/** Description, as XML name (without special characters).  
		 * @since wiki2xhtml 3.3 */
		public static final String descXmlName = "[descxmlname]";
		public static final String id = "[id]";

		public static final String meta = "[meta]";
		public static final String title = "[title]";
		public static final String head = "[head]";
		public static final Pattern regexHead = Resources.Regex.reckHead;
		public static final String textheader = "[textheader]";
		public static final String menu = "[menu]";
		public static final String footer = "[footer]";
		public static final String h1 = "[h1]";
		public static final String homelink = "[homelink]";
		public static final String content = "[content]";
		
	}
	
	public static final class Settings {
		
		public static final char separator = ':';
		public static final String arg = separator + "([^}]+)";
		
		// Also CSS settings
		public static final String galleryImagesPerLine = "GalleryImagesPerLine";
		public static final String galleryThumbWidth = "GalleryThumbWidth";
		public static final String imagepageImgWidth = "ImageWidthImagepage";
		public static final String thumbWidth = "ThumbWidthImages";
		
		// Only page settings
		public static final String author = "Author";
		public static final String consistentGallery = "ConsistentGallery";
		public static final String descForCaption = "DescForCaption";
		public static final String desc = "Desc(?:ription)?";
		public static final String defaultTitle = "DefaultTitle";
		public static final String h1 = "H1";
		public static final String homelink = "Homelink";
		public static final String icon = "Icon";
		public static final String imagepagesDir = "DirImagepages";
		public static final String imagesDir = "DirImages";
		public static final String keywords = "Keywords";
		public static final String keywordsAdd = "AddKeywords";
		public static final String lang = "Lang";
		public static final String meta = "Meta";
		public static final String nameForCaption = "NameForCaption";
		public static final String reckAlternative = "ReckAlternative";	// @since 3.3.2
		public static final String textHeader = "TextHeader";
		public static final String thumbsDir = "Thumbnails";
		public static final String title = "Title";
		
		/**
		 * @since 3.1.1
		 */
		public static final class Args {
			// Standard arguments
			public static final String argFalse = ":false";	
		} 
		
	}
	
	/** Tags which will be replaced in the document, e.g. by the current version */
	public static final class Tags {

		public static final String anchor = "~~(.+?)~~";
		public static final String splitPageNav = "{{$SplitPageNav}}";
		public static final String version = "{{$Version}}";
		public static final String top = "{{Top}}";
		public static final Pattern regexToc = Resources.Regex.toc;
		public static final String pagename = "{{$Pagename}}";
		public static final String wiki2xhtml = "{{$wiki2xhtml}}";
		
		public static final class Title {
			
			public static final String title = "%s";
			public static final String page = "%p";
			
		}
		
	}
	
	public static final class Placeholder {
		
		public static final String toc = "__TOC__";
		
	}
	
	public static final class ProgramSettings {
		
		public static final String autoUpdate = "AutoUpdateCheck"; 
		public static final String lastArgs = "LastArguments";
		public static final String locale = "Locale";
		public static final String ignoreNewVersion = "IgnoreNewVersion";
		
	}
	
	public static final class Keys {
		
		public static final KeyStroke quit = KeyStroke.getKeyStroke(KeyEvent.VK_Q, KeyEvent.CTRL_DOWN_MASK);
		public static final KeyStroke close = KeyStroke.getKeyStroke(KeyEvent.VK_W, KeyEvent.CTRL_DOWN_MASK);
		public static final KeyStroke esc = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		public static final KeyStroke ctrlNext = KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, KeyEvent.CTRL_DOWN_MASK);
		public static final KeyStroke ctrlPrev = KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, KeyEvent.CTRL_DOWN_MASK);
		public static final KeyStroke ctrlPgDown = KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, KeyEvent.CTRL_DOWN_MASK);
		public static final KeyStroke ctrlPgUp = KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, KeyEvent.CTRL_DOWN_MASK);
		
		public static final KeyStroke toggleCodeWindow = KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.ALT_DOWN_MASK);
		
	}
	
	public static final class Filters {
		
		public static final FileFilter designFiles = new FileFilter() {

			@Override
			public String getDescription() {
				return "All needed files for a design";
			}

			@Override
			public boolean accept(File f) {
				String name = f.getName().toLowerCase();
				return name.endsWith(".css") || name.endsWith(".png")
						|| name.endsWith(".jpg") || name.endsWith(".jpeg") || name.endsWith(".ico") 
						|| name.endsWith(".gif") || name.endsWith("xsl");
			}

		};
		
		public static final FileFilter copyOnly = new FileFilter() {
			@Override
			public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				String name = f.getName();
				return 
					name.endsWith(".html") 
					|| name.endsWith(".htm")
					;
			}
			
			@Override
			public String getDescription() {
				return "Files which will be copied only and not processed by wiki2xhtml.";
			}
		};
		
		public static final FileFilter scriptFile = new FileFilter() {
			@Override
			public boolean accept(File f) {
				Matcher m = Resources.Regex.scriptModeFile.matcher(f.getName());
				return m.find();
			}
			@Override
			public String getDescription() {
				return "Script file, e.g. php or asp";
			}
		};
		
	}
	
	/** Tags used in the updater file */
	public static final class UpdaterTags {
		
		public static final String currentVersion = "CurrentVersion";
		public static final String currentDevVersion = "CurrentDevVersion";
		public static final String currentVersionDate = "CurrentVersionDate";
		
		public static final String versionNotes = "VersionNotes";
		
		public static final String increaseSizeBy = "IncreaseSizeBy";
		
		public static final String[] infos = new String[] { currentVersion, currentVersionDate, currentDevVersion, versionNotes };
		public static final String[] all = new String[] { currentVersion, currentVersionDate, currentDevVersion, versionNotes, increaseSizeBy };
		
	}
	
}
