package program;

import java.io.File;

import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

import program.commentator.CommentAtor;
import program.update.UpdateChecker;
import program.utilities.StringTools;



/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Handles arguments from the command line
 * 
 * TODO 5 setting to compile changed files only
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class Handler_Arguments {
	
	private static final I18n in = I18nFactory.getI18n(Handler_Arguments.class, "bin.l10n.Messages", program.Globals.getLocale());

	public static StringBuffer completeSummary = new StringBuffer();

	final static String info = in.tr("Info") + ": ";

	final static String warn = in.tr("WARNING") + ": ";

	public static void printHelp() {
		CommentAtor.getInstance().ol(Help.getHelpText(0, "\n   ", "\n"), CommentAtor.ANY);
	}

	private static void noteEq(short i, String arg) {
		CommentAtor.getInstance().ol(warn + in.tr("No file given for the argument Nr. ") + i + " (" + arg + "). "
				+ in.tr("Maybe you set a space before the filename?"), CommentAtor.MSG);
	}

	private static void noteEqD(short i, String arg) {
		CommentAtor.getInstance().ol(warn + in.tr("No directory given for the argument Nr. ") + i + " (" + arg + "). "
				+ in.tr("Maybe you set a space before the filename?"), CommentAtor.MSG);
	}

	private static void noteForgot(short i, String arg, String what) {
		CommentAtor.getInstance().ol(warn + in.tr("You forgot the ") + what + ' ' + in.tr("to the argument ") + arg + " ("
				+ in.tr("Nr. ") + (i) + ")", CommentAtor.MSG);
	}

	private static final void obsolete(String s) {
		CommentAtor.getInstance().ol(s + ": " + in.tr("This argument is obsolete and no longer supported. Please use -c instead!") + '\n'
						+ ConstantTexts.seeHelp, CommentAtor.ANY);
	}
	
	public static String handleReplaceArguments(String arguments) {

		/*
		 * *********************************************************************
		 * **** Handle the arguments --helpfile(s) and --standard
		 * *********************************************************************
		 */
		
		if (arguments.contains("--helpfiles-de")) {

			StringBuffer sb = new StringBuffer(
					Constants.Arguments.stdArgsHelpDe.getArgs().append(arguments) 
					+ " ");

			StringTools.replaceAll(sb, "--helpfiles-de", "");
			StringTools.replaceAll(sb, "  ", " ");

			arguments = sb.toString();
		}
		if (arguments.contains("--helpfiles")) {

			StringBuffer sb = new StringBuffer(
					Constants.Arguments.stdArgsHelpEn.getArgs().append(arguments) 
					+ " ");

			StringTools.replaceAll(sb, "--helpfiles", "");
			StringTools.replaceAll(sb, "  ", " ");

			arguments = sb.toString();
		}
		if (arguments.contains("--standard")) {

			StringBuffer sb = StringTools.replaceAll(
					new StringBuffer(Constants.Arguments.stdArgs.getArgs() + arguments + " "), 
					"--standard", "");
			StringTools.replaceAll(sb, "  ", " ");

			arguments = sb.toString();
		}
		return arguments;
	}

	/**
	 * Sets up the CommentAtor and the File
	 * @param args
	 * @param execute
	 * @return true, if wiki2xhtml has to be run (GUI not started)
	 */
	public static boolean handleCommandLineArguments(Args args, boolean execute) {

		Container_Files fc = Container_Files.getInstance();
		fc.clear();	// Reset all settings
		
		if (args.size() == 0) {
			if (execute) {
				/*
				 * No arguments given, so try to open GUI
				 */
				CommentAtor.getInstance().ol(in.tr("Opening GUI."), CommentAtor.MSG);
				
				UserInterface.runGui(null, false);
				
				return false;
			}
		} else {
			
			String arg;
			int l = args.size();
			boolean alreadyHandled = false;
			boolean gui = false;
			
			for (short i = 0; i < args.size(); i++) {
				
				if (alreadyHandled) {
					// Argument has already been handled, e.g. because it was a file.
					alreadyHandled = false;
					continue;
				}
				
				arg = args.getPlain(i);
				
				if (new File(arg).exists()) {
					fc.addFile(arg);
					continue;
				}
				
				/* Flags */
				
				if (arg.equals(Constants.Arguments.Flags.dead)) {
					CommentAtor.getInstance().setDead();
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.debug)) {
					CommentAtor.getInstance().setDebug();
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.gui)) {
					gui = true;
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.help)) {
					printHelp();
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.helpfiles))
					continue;	// handled by handleReplaceArguments()
				
				if (arg.equals(Constants.Arguments.Flags.helpfilesDe))
					continue;
				
				if (arg.equals(Constants.Arguments.Flags.helpLong)) {
					printHelp();
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.noupdatecheck)) {
					fc.sc.noUpdateCheck = true;
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.onlyCode)) {
					fc.sc.onlyCode = true;
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.removeLinebreaks)) {
					fc.sc.removeLineBreaks = true;
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.silent)) {
					CommentAtor.getInstance().setSilent();
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.standard))
					continue;
				
				if (arg.equals(Constants.Arguments.Flags.stdout)) {
					fc.sc.stdOut = true;
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.typos)) {
					CommentAtor.getInstance().ol(Constants.Arguments.Flags.typos + ": " + ConstantTexts.notYetSupported, CommentAtor.MSG);
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.updateCheck)) {
					System.out.println();
					System.out.println(UpdateChecker.getUpdateMessage());
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Flags.verbose)) {
					CommentAtor.getInstance().setVerbose();
					continue;
				}
				
				
				/* Args */
				
				if (arg.equals(Constants.Arguments.Arg.common)) {
					if (i+1 < l) {
						fc.setCommonFile(args.getPlain(i+1));
						alreadyHandled = true;
					} else {
						noteForgot(i, arg, ConstantTexts.NounCap.commonFile);
					}
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Arg.footer)) {
					if (i+1 < l) {
						fc.setFooterFile(args.getPlain(i+1));
						alreadyHandled = true;
					} else {
						noteForgot(i, arg, ConstantTexts.NounCap.footerFile);
					}
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Arg.menu)) {
					if (i+1 < l) {
						fc.setMenuFile(args.getPlain(i+1));
						alreadyHandled = true;
					} else {
						noteForgot(i, arg, ConstantTexts.NounCap.menuFile);
					}
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Arg.style)) {
					if (i+1 < l) {
						fc.setStyleDirectory(args.getPlain(i+1));
						alreadyHandled = true;
					} else {
						noteForgot(i, arg, ConstantTexts.styleDir);
					}
					continue;
				}
				
				if (arg.equals(Constants.Arguments.Arg.targetDir)) {
					if (i+1 < l) {
						fc.setTargetDirectory(args.get(i+1));
						alreadyHandled = true;
					} else {
						noteForgot(i, arg, ConstantTexts.targetDir);
					}
					continue;
				}
				
				
				/* Combined Args */
				
				if (arg.startsWith(Constants.Arguments.CombinedArg.footer)) {
					if (arg.length() > Constants.Arguments.CombinedArg.footer.length()) {
						fc.setFooterFile(arg.substring(arg.indexOf('=')+1));
					} else {
						noteEq(i, args.get(i));
					}
					continue;
				}
				
				if (arg.startsWith(Constants.Arguments.CombinedArg.lang)) {
					if (arg.length() > Constants.Arguments.CombinedArg.lang.length()) {
						program.Globals.setLocale(arg.substring(arg.indexOf('=')+1));
						CommentAtor.getInstance().ol(in.tr("Language is now ") + program.Globals.getLocale(), CommentAtor.DEBUG);
					} else {
						noteEq(i, args.get(i));
					}
					continue;
				}
				
				if (arg.startsWith(Constants.Arguments.CombinedArg.menu)) {
					if (arg.length() > Constants.Arguments.CombinedArg.menu.length()) {
						fc.setMenuFile(arg.substring(arg.indexOf('=')+1));
					} else {
						noteEq(i, args.get(i));
					}
					continue;
				}
				
				if (arg.startsWith(Constants.Arguments.CombinedArg.style)) {
					if (arg.length() > Constants.Arguments.CombinedArg.style.length()) {
						fc.setStyleDirectory(arg.substring(arg.indexOf('=')+1));
					} else {
						noteEqD(i, args.get(i));
					}
					continue;
				}
				
				if (arg.startsWith(Constants.Arguments.CombinedArg.targetDir)) {
					if (arg.length() > Constants.Arguments.CombinedArg.targetDir.length()) {
						fc.setTargetDirectory(arg.substring(arg.indexOf('=')+1));
					} else {
						noteEqD(i, args.get(i));
					}
					continue;
				}
				
				
				/* Obsolete */
				
				boolean obsolete = false;
				for (String s : Constants.Arguments.obsoleteFlags) {
					if (arg.equals(s)) {
						obsolete = true;
						obsolete(args.get(i));
						break;
					}
				}
				if (obsolete) continue;
				
				for (String s : Constants.Arguments.obsoleteCombined) {
					if (arg.equals(s)) {
						obsolete = true;
						obsolete(args.get(i));
						break;
					}
				}
				if (obsolete) continue;
				
				for (String s : Constants.Arguments.obsoleteArgs) {
					if (arg.equals(s)) {
						obsolete = true;
						alreadyHandled = true;
						obsolete(args.get(i));
						break;
					}
				}
				if (obsolete) continue;
				
				CommentAtor.getInstance().ol(warn + in.tr("The argument ") + arg
						+ in.tr(" could not be recognized. ") + ConstantTexts.seeHelp, CommentAtor.ERRORS);
				
			}
			
			StringBuffer fileList;
			if ((fileList = fc.getFileList().getArgs()).length() > 0) {
				CommentAtor.getInstance().ol(ConstantTexts.files + ": " + fileList, CommentAtor.MSG);
			}
			
			if (gui && execute) {
				UserInterface.runGui(null, false);	//TOOD 0 test
				return false;
			}
			
		}
		
		return true;
	}
}
