package program;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

import program.commentator.CommentAtor;
import program.commentator.Logger;
import program.pagesplitter.SplitPageContainer;
import program.pagesplitter.SplitPageLinks;
import program.utilities.IORead_Stats;
import program.utilities.StringTools;

import regexp.Regex;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Replaces tags by various methods
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class ReplaceTags {
	
	private static I18n i18n = I18nFactory.getI18n(ReplaceTags.class, "bin.l10n.Messages", program.Globals.getLocale());
	private static final CommentAtor ca = CommentAtor.getInstance();
	
	StringBuffer content = new StringBuffer(), toc = new StringBuffer();
	
	public ReplaceTags() {	}
	
	public ReplaceTags(StringBuffer in, StringBuffer TOC) {
		content = in;
		toc = TOC;
	}
	
	public void replaceAll() {
		replaceVersion(content);
		replaceTop();
		replaceTOC();
		replacePagename();
		replaceSplitPageLinks();
		replaceWiki2xhtml(content);
	}
	public static StringBuffer replaceVersion(StringBuffer content) {
		return StringTools.replaceAll(content, Constants.Tags.version, Constants.Wiki2xhtml.version);
	}
	public static StringBuffer replaceWiki2xhtml(StringBuffer content) {
		return StringTools.replaceAll(content, Constants.Tags.wiki2xhtml, Constants.Wiki2xhtml.webpage);
	}
	
	/**
	 * Replaces Top links with either a template or a pre-defined tag
	 */
	private void replaceTop() {
		File t = new File(Container_Files.getInstance().cont.styleDir.getPath() + File.separatorChar + "toplink.template");
		StringBuffer replace = new StringBuffer();
		if (t.exists() && t.isFile()) {
			try { replace = new StringBuffer(IORead_Stats.readSBuffer(t)); } 
			catch (IOException e) {
				Logger.getInstance().log.append(e);
				e.printStackTrace();
			}
		} else
			replace = new StringBuffer("<span class=\"toplink\" ><a href=\"#content\">Top</a></span>");
		StringTools.replaceAll(content, Constants.Tags.top, replace.toString());
	}

	/**
	 * Inserts the TOC (only once)
	 * @since 3.3.2
	 */
	private void replaceTOC() {
		java.util.regex.Matcher m = Constants.Tags.regexToc.matcher(content);
		if (m.find()) {
			// Replace TOC Heading
			String s = toc.toString().replace(Constants.Placeholder.toc, 
					(m.group(1) != null && m.group(1).length() > 0 ? m.group(1) : ConstantTexts.toc));
			content.replace(m.start(), m.end(), s);
		}
	}
	
	private void replacePagename() {
		StringTools.replaceAll(content, Constants.Tags.pagename, Container_Files.getInstance().currentFilename);
	}
	
	/**
	 * Inserts the navigation between split pages 
	 */
	private void replaceSplitPageLinks() {
		if (SplitPageContainer.getInstance().isSplit) {
			StringTools.replaceAll(
					content, 
					Constants.Tags.splitPageNav, 
					SplitPageLinks.getInstance().getLinkList()
				);
		} else {
			StringTools.replaceAll(content, Constants.Tags.splitPageNav, "");
		}
	}
	
	/**
	 * Searches for ~~name~~ and replaces it with <a id="name"></a>.
	 */
	public void replaceMarks() {
		StringBuffer funcName = new StringBuffer("ReplaceTags.replaceMarks(): ");
		Regex rex = new Regex(Constants.Tags.anchor, content.toString());
		
		if (rex.found) {
			ArrayList<StringBuffer> checklist = new ArrayList<StringBuffer>();
			StringBuffer name = new StringBuffer();
			StringBuffer out = new StringBuffer();
			int last = 0, first;
			while (rex.find()) {
				first = rex.start();
				out.append(content.subSequence(last, first));
				last = rex.end();
				
				name = new StringBuffer(rex.group(1));
				if (checklist.contains(name))
					ca.ol(funcName + i18n.tr("Warning: Duplicate mark!") + " (" + name + ") " + i18n.tr("Mark removed."), CommentAtor.ERRORS);
				else if (name.indexOf("\"") >= 0) {
					ca.ol(funcName + i18n.tr("Warning: Mark must not contain such a character!") 
							+ " (\") " + i18n.tr("Mark removed."), CommentAtor.MSG);
				} else {
					checklist.add(name);
					out.append("<a id=\"");
					out.append(name);
					out.append("\"></a>");
				}
				
			}
			out.append(content.subSequence(last, content.length()));
			content = out;
		}
	}
	
}
