package program;

import java.awt.Point;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Calendar;

import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

import program.commentator.CommentAtor;
import program.commentator.Logger;
import program.pagesplitter.SplitPageContainer;
import program.pagesplitter.SplitPageLinks;
import program.settings.XhtmlSettings;
import program.settings.XhtmlSettingsReader;
import program.tools.UpdateMenufile;
import program.tools.UpdateTemplate;
import program.utilities.IORead_Stats;
import program.utilities.IOUtils;
import program.utilities.IOWrite_Stats;
import program.utilities.LanguageTools;
import program.utilities.StringTools;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * The main file, starts wiki2xhtml.
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * 
 */
public class UserInterface {
	
	private static I18n i18n = I18nFactory.getI18n(UserInterface.class, "bin.l10n.Messages", program.Globals.getLocale());
	private static CommentAtor ca = CommentAtor.getInstance();

	static short fileNumber, nFiles;
	static int progress = 0;

	public static void restartGui() {
		Point p = Globals.getGuiManager().f.getLocation();
		
		Globals.getGuiManager().f.setVisible(false);
		Globals.getGuiManager().f.dispose();
		Globals.guiManager = null;
		Globals.guiManager = new GUI();
		
		runGui(p, true);
		Globals.getGuiManager().updateSettings();
	}
	
	public static void runGui(Point p, boolean wasRestarted) {
		Globals.getGuiManager().gui(wasRestarted);
		Globals.getGuiManager().f.setVisible(true);
		if (p != null)
			Globals.getGuiManager().f.setLocation(p);
	}

	public static void main(String cmdLineArgs[]) {
		
		if (Globals.programSettings.readSettings()) {
			program.Globals.setLocale(Globals.programSettings.settings.get(Constants.ProgramSettings.locale));
		}
		
		/*
		 * Run the menu file updater if wished
		 */
		if (cmdLineArgs.length > 0 && Constants.Arguments.Special.menufileUpdater.equals(cmdLineArgs[0])) {
			if (cmdLineArgs.length > 1) {
				UpdateMenufile.run(cmdLineArgs[1], true);
			} else
				UpdateMenufile.run(null, true);
			
			System.exit(0);
		}
		
		/*
		 * Run the reck updater if desired
		 */
		if (cmdLineArgs.length > 0 && Constants.Arguments.Special.templateUpdater.equals(cmdLineArgs[0])) {
			if (cmdLineArgs.length > 1) {
				UpdateTemplate.run(cmdLineArgs[1], true);
			} else {
				UpdateTemplate.run(null, true);
			}
			
			System.exit(0);
		}
		
		/*
		 * Or just run wiki2xhtml
		 */
		UserInterface nonStatic = new UserInterface();
		
		if (StringTools.stringArrayToSB(cmdLineArgs, " ").indexOf("--dead") < 0 && cmdLineArgs.toString().indexOf("--silent") < 0)
			ca.ol("wiki2xhtml " + Constants.Wiki2xhtml.version, CommentAtor.MSG);
		
		StringBuffer sb = new StringBuffer();
		for (String s : cmdLineArgs)
			if (s.contains(" "))
				sb.append(" \"" + s + "\"");
			else
				sb.append(' ' + s);
		
 		Args a = new Args(Handler_Arguments.handleReplaceArguments(sb.toString()));

		/*
		 * Standard settings
		 */

		if (Handler_Arguments.handleCommandLineArguments(a, true))
			// GUI has not been opened, so run the parser
			nonStatic.make();
		
	}

	public void make() {
		
		Container_Files fc = Container_Files.getInstance();
		
		if (fc.getFilesNumber() > 0) {
			
			int files = 0, charsRead = 0, charsWritten = 0;
			
			Statistics.getInstance().reset();
			
			Statistics.getInstance().sw.timeOverall.start();
			Statistics.getInstance().sw.timeInitializingProcess.start();
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			/** will contain the result */
			StringBuffer xhtmlOutput = new StringBuffer();
			/** Contains the menu */
			WikiMenu wm = new WikiMenu();

			File in = null; // The source file
			File out = null; // The target file


			ca.ol(XHTML.PC(), CommentAtor.V_MSG);

			/*
			 * DESIGN use random style if none was selected
			 */
			if (!fc.has.style)
				if (XHTML.baseStyleDir.exists() && XHTML.baseStyleDir.isDirectory()) {
					File[] sf = XHTML.baseStyleDir.listFiles(new FilenameFilter() {
						public boolean accept(File arg0, String arg1) {
							return new File(XHTML.baseStyleDir.getPath()
									+ File.separatorChar + arg1).isDirectory()
									&& !".svn".equals( arg1 );
						}
					});
					if (sf.length > 0)
						fc.setStyleDirectory(sf[(int) (Math.random() * sf.length)].getPath());
				}
			

			/*
			 * Read design specific settings
			 */
			XhtmlSettingsReader.getDesignSettings(Container_Files.getInstance().cont.cssSettings());

			/*
			 * COMMON read common settings
			 */
			if (fc.has.common) {
				try {
					XhtmlSettingsReader.getSettings(IORead_Stats.readSBuffer(fc.cont.commonFile));
				} catch (IOException e) {
					CommentAtor.getInstance().ol("Could not write file " + fc.cont.commonFile.getAbsolutePath(), CommentAtor.ERRORS);
					Logger.getInstance().log.append(e);
					e.printStackTrace();
				}
			}

			/*
			 * FOOTER set the footer
			 */
			if (fc.has.footer) {
				try {
					xhs.global.footer = IORead_Stats.readSBuffer(fc.cont.footerFile);
				} catch (IOException e) {
					CommentAtor.getInstance().ol("Could not read file " + fc.cont.footerFile.getAbsolutePath(), CommentAtor.ERRORS);
					Logger.getInstance().log.append(e);
					e.printStackTrace();
				}
				StringTools.replaceAll(xhs.global.footer, "%v", Constants.Wiki2xhtml.versionNumber);
				StringTools.replaceAll(xhs.global.footer, "%d", Calendar.getInstance().getTime().toString());
			}

			/*
			 * MENU set the menu
			 */
			if (fc.has.menu) {
				try {
					wm.readNewMenu(fc.cont.menuFile);
				} catch (IOException e) {
					Logger.getInstance().log.append(e);
					e.printStackTrace();
				}
			}

			/*
			 * DESIGN copy the necessary files
			 */
			IOUtils.copyDirectory(fc.cont.styleDir, fc.cont.targetSDir(), Constants.Filters.designFiles,
					(ca.flags & CommentAtor.V_MSG) == 0, 
					(ca.flags & CommentAtor.V_MSG) == 0);

			/*
			 * PARSE look how many files have to be parsed
			 */
			setNFiles(fc.getFilesNumber());
			
			/* ***** ***** ***** ***** ***** 
			 * Handle the files  ***** ***** 
			 * ***** ***** ***** ***** *****  
			 */
			
			String filenameNE = "", extension = "";
			
			boolean nowiki;
			SplitPageContainer spc = SplitPageContainer.getInstance();
			SplitPageLinks spl = SplitPageLinks.getInstance();
			
			Statistics.getInstance().sw.timeInitializingProcess.stop();

			if (!fc.sc.stdOut) 
				ca.ol(ConstantTexts.styleDir + ": " + fc.cont.styleDir.getPath(), CommentAtor.MSG);
			
			for (short i = 0; i < nFiles; i++) {
				setFileNumber(i);

				in = fc.getFile(i);
				
				// Don't process file if it doesn't need to (.html)
				if (Constants.Filters.copyOnly.accept(in)) {
					IOUtils.copyFile(in, new File(
							Container_Files.getInstance().cont.targetDir().getPath() 
							+ File.separatorChar + in.getName()));
					CommentAtor.getInstance().ol("Copied: " + in.getPath(), CommentAtor.V_MSG);
					continue;
				}
				
				try {
					spc.setSource(IORead_Stats.readSBuffer(in));
				} catch (IOException e) {
					Logger.getInstance().log.append(e);
					e.printStackTrace();
				}
				spc.splitPages();
				charsRead += spc.getCharNumber();
				
				/*
				 * Set the file extension for the output file
				 */
				
				nowiki = false;
				if (!fc.sc.stdOut) {
					filenameNE = IOUtils.removeFileExtension(in).getName();
					
					if (Constants.Filters.scriptFile.accept(in)) {
						
						extension = in.getName().substring(in.getName().lastIndexOf('.'));
						xhs.local.scriptMode = true;
						
					} else
						extension = ".html";
				}
				
				spl.setup(filenameNE, extension);

				for (int p = spc.pages.size() - 1; p >= 0; p--) {
					if (spc.currentPageNumber > 1)
						out = new File(fc.cont.targetDir().getPath() + XHTML.fileSep + filenameNE + spc.currentPageNumber + extension);
					else
						out = new File(fc.cont.targetDir().getPath() + XHTML.fileSep + filenameNE + extension);
					
					fc.currentFilename = out.getName();
					xhs.local.content = new StringBuffer(spc.pages.get(p));
					xhs.setPageAddon((byte) (spc.currentPageNumber), (byte) spc.pages.size());
					
					if (!fc.sc.stdOut)
						ca.o(out.getPath() + " ... ", CommentAtor.MSG);
					
					if (xhs.local.content.indexOf("<!--nowiki-->") >= 0)
						nowiki = true;
					
					/*
					 * SCRIPT Cut the script at the top away, if there is one (php)
					 */
	
					{
						int end;
						if ((end = xhs.local.content.indexOf("<!--endtop-->")) > 0) {
							xhs.local.topScript = new StringBuffer(xhs.local.content.subSequence(0, end));
							xhs.local.content.delete(0, end + "<!--endtop-->".length());
						}
					}
	
					/*
					 * MENU activates the menu entry
					 */
					wm.openItem(filenameNE + extension, WikiMenu.LINK);
					xhs.local.menu = wm.getMenu();
	
					if (nowiki)
						xhtmlOutput = xhs.local.content;
					else {
						/*
						 * Convert the file
						 */
						xhs.getHead();
						ReplaceTags rt;
						rt = new ReplaceTags();
						rt.content = xhs.local.content;
						rt.replaceMarks();
						xhs.local.content = rt.content;
		
						xhs.local.content = new StringBuffer(XHTML.wiki2xhtml());
						xhs.buildMetadata();
						WikiHeadings.getTOC(xhs.local.content);
						
						// Add the page navigator, if the page is split
						spl.setLang(LanguageTools.getLocale(LanguageTools.getLanguage(xhs.getLang().toString())));
						if (spc.pages.size() > 1) //TODO 1 only if no tag used
							xhs.local.content.append(spl.getLinkList());
						
						
		
						rt = new ReplaceTags(xhs.local.content, xhs.local.toc);
						rt.replaceAll();
						xhs.local.content = rt.content;
		
						WikiReferences refs = new WikiReferences();
						refs.content = xhs.local.content;
						refs.makeReferences();
						if (refs.refPlaceFound)
							xhs.local.content = refs.content;
		
						if (!fc.sc.onlyCode)
							try {
								xhtmlOutput = new StringBuffer(XHTML.xhtmlToFile());
							} catch (IOException e) {
								Logger.getInstance().log.append(e);
								e.printStackTrace();
								xhtmlOutput = xhs.local.content;
							}
						else
							xhtmlOutput = xhs.local.content;
					}
					
					/*
					 * Write the file
					 */
					if (fc.sc.stdOut)
						ca.ol(xhtmlOutput, CommentAtor.ANY);
					else {
						files++;
						charsWritten += out.length();
						try {
							IOWrite_Stats.writeString(out, xhtmlOutput.toString(), false);
						} catch (IOException e) {
							Logger.getInstance().log.append(e);
							e.printStackTrace();
						}
						ca.ol(i18n.tr("done"), CommentAtor.MSG);
						
						ca.ol("\n---------------\n" + in.getPath() + " > " + out.getPath() + '\n'
								+ i18n.tr("Title") + ": " + xhs.local.title
								+ "\n---------------\n" + '\n', CommentAtor.V_MSG);
					}
					spc.currentPageNumber++;
					
				}
				xhs.init();
			}
			
			Statistics.getInstance().sw.timeOverall.stop();
			

			ca.ol(i18n.tr("Files written: " + files + "; characters read/written: " + charsRead + " / " + 
					charsWritten + ( charsRead > 0 ? " (" + (100 * charsWritten / charsRead) + " %)" : "" ) + 
					"\nTime taken: " + Statistics.getInstance().sw.timeOverall.getStoppedTimeString()), (byte) (CommentAtor.MSG | CommentAtor.V_MSG));
			
			CommentAtor.getInstance().ol(Statistics.getInstance().generateStats(), CommentAtor.MSG);

			if (Statistics.getInstance().sw.temp.getStoppedTime() > 0) {
				System.err.println(Statistics.getInstance().sw.temp.getStoppedTimeString());
			}
			if (Statistics.getInstance().sw.temp2.getStoppedTime() > 0) {
				System.err.println(Statistics.getInstance().sw.temp2.getStoppedTimeString());
				
			}
			
		}
		
	}

	public static void setFileNumber(short i) {
		fileNumber = (short) (i + 1);
	}

	public static void setNFiles(short i) {
		nFiles = i;
	}

	public static void setProgress(int i) {
		if (nFiles != 0) {
			float p = (float) (fileNumber - 1) / nFiles * 100 + (i / nFiles);
			progress = Math.round(p);
			try {
				Globals.getGuiManager().setProgress(progress);
				Globals.getGuiManager().setTime(Statistics.getInstance().sw.timeOverall.getPassedTime());
			} catch (NullPointerException e) {}
		}
	}
}