package program;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import program.commentator.CommentAtor;
import program.settings.XhtmlSettings;
import program.settings.ImageSettings.Specific;
import program.typo.Formattings;
import program.utilities.IORead_Stats;
import program.utilities.IOWrite_Stats;
import program.utilities.StringTools;
import program.utilities.XMLTools;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Generates html code for images 
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * 
 * TODO 2 style: breite von thumbnails in css dynamisch
 */
public class WikiImages {
	
	private static final CommentAtor ca = CommentAtor.getInstance();
	
	private static final program.settings.ImageSettings is = program.settings.ImageSettings.getInstance();
	
	/**
	 * Generates a thumbnail entry.
	 * @return Thumbnail code
	 */
	public static StringBuffer generateThumbnailEntry() {
		String link;
		
		if (is.specific.isImagepageCreated) {
			link = is.specific.getImagepagePath(false, false);
		} else {
			link = is.specific.getImagePathHtml();
		}
		
		Template tp = is.specific.getTemplate();
		tp.clear();
		tp.replaceAll(Constants.TemplateTags.pos, is.specific.position);
		tp.replaceAll(Constants.TemplateTags.link, link);
		tp.replaceAll(Constants.TemplateTags.thumb, is.specific.getThumbSrc());
		tp.replaceAll(Constants.TemplateTags.imagePath, is.specific.getImagePathHtml());
		tp.replaceAll(Constants.TemplateTags.width, is.specific.getWidthThumbAttrib());
		tp.replaceAll(Constants.TemplateTags.desc, (is.specific.text.length() > 0 ? is.specific.text : is.specific.desc.toString()));
		tp.replaceAll(Constants.TemplateTags.descXmlName, XMLTools.getXmlNameChar(is.specific.text.length() > 0 ? is.specific.text : is.specific.desc.toString(), "\\s"));
		tp.replaceAll(Constants.TemplateTags.id, is.specific.getID());

		return tp.getOutput();
	}

	/**
	 * Make the [[Image:xx]] to &lt;img src="xx" /&gt; tags.
	 * <p>Rewritten: August 2008</p>
	 * 
	 * @param in The input file
	 * @return input file with replaced image tags
	 */
	public static StringBuffer makeImages(StringBuffer in) {
		Statistics.getInstance().sw.timeInsertingImages.continueTime();
		
		Container_Files fc = Container_Files.getInstance();

		StringBuffer out = new StringBuffer();
		short counter = 0;
		
		Matcher m;

		is.nextPage(fc.currentFilename);
		
		m = Resources.Regex.images.matcher(in);
		int last = 0;
		
		while (m.find()) {
			out.append(in.substring(last, m.start()));
			Statistics.getInstance().counter.imagesTotal.increase();
			
			// Read the image's arguments
			is.nextItem();
			is.specific.readArguments(m.group(1), false);
			is.specific.isGallery = false;
			
			// Generate an image page if they are to be generated, a isThumbDesired has to be inserted and it's no direct link. 
			if (is.specific.isThumbDesired && !is.specific.direct) {
				generateImagepage();
			}
			
			out.append(generateThumbnailEntry());
			

			last = m.end();
		}
		out.append(in.substring(last, in.length()));

		/* Statistics */
		Statistics.getInstance().counter.imagesTotal.increase(counter);
		Statistics.getInstance().sw.timeInsertingImages.stop();
		
		return out;
	}
	
	/**
	 * Builds all galleries
	 * @param in File input
	 * @return
	 */
	public static StringBuffer makeGallery(StringBuffer in) {
		Statistics.getInstance().sw.timeInsertingImages.continueTime();

		ArrayList<NavLinks> entries = new ArrayList<NavLinks>();
		
		BufferedReader b = new BufferedReader(new StringReader(in.toString()));
		StringBuffer out = new StringBuffer();
		
		Template tp = new Template(
						new File(Container_Files.getInstance().cont.styleDir.getAbsolutePath() + File.separatorChar + "galleryContainer.html"), 
						Container_Resources.readResource(Container_Resources.sgalleryContainer)
					);
		
		byte counter = 0;
		
		try {
			String line;
			String caption = new String();
			
			Specific entry;
			
			Pattern pImageLine = Pattern.compile("(?:Image|Bild):(.*)");
			Matcher mImagesLine;
			
			Pattern pCaption = Pattern.compile("caption=\"([^\"]*)\"");
			Matcher mCaption;
			
			boolean open = false;
			
			for (line = b.readLine(); line != null; line = b.readLine()) {
				
				if (open) {
					if (line.startsWith("</gallery>")) {
						// Add all entries, generate image pages
						
						open = false;
						
						tp.clear();
						
						// Linking items
						for (int i = 0; i < entries.size(); i++) {
							
							try { entries.get(i).setPrevNL(entries.get(i-1)); }
							catch (IndexOutOfBoundsException e) { }
							
							try { entries.get(i).setNextNL(entries.get(i+1)); }
							catch (IndexOutOfBoundsException e) { }
						}
						
						// Set up links and generate image pages and thumbnail entries
						byte gipl = XhtmlSettings.getInstance().local.getGalleryImagesPerLine();
						for (int i = 0; i < entries.size(); i++) {

							is.specific = entries.get(i).item;
							
							entries.get(i).item.nextLink = entries.get(i).getLinkToOther(false);
							entries.get(i).item.prevLink = entries.get(i).getLinkToOther(true);
							
							if (entries.get(i).item.text.length() == 0 && !is.specific.direct)
								generateImagepage();
							
							tp.content.append(generateThumbnailEntry());
							
							// Add new image line if wished
							if (is.pagespecific.useConsistentGallery() 
									&& i > 1
									&& (i+1) % gipl == 0)
								tp.content.append("\n<br style=\"clear: both;\" />");
							
						}
						
						tp.replace(Constants.TemplateTags.caption, "<h3>" + caption + "</h3>");
						tp.replace(Constants.TemplateTags.id, is.pagespecific.getIdGallery());
						tp.replaceContent();
						
						out.append(tp.getOutput());
						entries.clear();	// Remove all entries
						is.pagespecific.counterGalleries++;
						
					} else {
						
						// Add item
						
						entry = new Specific();
						
						mImagesLine = pImageLine.matcher(line);
						if (mImagesLine.find()) {
							entry.readArguments(mImagesLine.group(1), false);
						} else {
							entry.readArguments(line, true);
						}

						entry.isGallery = true;
						
						entries.add(new NavLinks(entry));
					}
				} else {
					if (line.startsWith("<gallery ") || line.startsWith("<gallery>")) {
						// add gallery start
						open = true;
						mCaption = pCaption.matcher(line);
						if (mCaption.find()) {
							caption = mCaption.group(1);
						} else
							caption = "";
						
					} else {
						// add line
						out.append(line + '\n');
					}
				}
				
				
			}
			
		} catch (IOException e) {
			e.printStackTrace();
			CommentAtor.getInstance().ol(
					String.format("IO Exception while trying to read content in file %s!", Container_Files.getInstance().currentFilename), 
					CommentAtor.ERRORS);
			return in;
		}

		/* Statistics */
		Statistics.getInstance().counter.galleries.increase(counter);
		Statistics.getInstance().sw.timeInsertingImages.stop();
		
		return out;
	}
	
	/**
	 * Generates an image page
	 */
	public static void generateImagepage() {
		
		try {
			StringBuffer sb = IORead_Stats.readSBuffer(Container_Files.getInstance().cont.imageTemplate());
			XhtmlSettings xhs = XhtmlSettings.getInstance();
			
			String caption;
			String desc = is.specific.desc.toString();
			if (is.specific.caption.length() > 0)
				caption = is.specific.caption;
			else {
				if (xhs.useImageDescAsCaption()) {
					caption = desc;
					desc = "";
				} else if (xhs.useImagenameAsCaption()){
					caption = is.specific.pathImage.substring(is.specific.pathImage.lastIndexOf('/'));
				} else
					caption = "";
			} 
	
			StringTools.replaceAll(sb, Constants.TemplateTags.imagePath, is.specific.getImagePathHtml());
			StringTools.replaceAll(sb, Constants.TemplateTags.imageName, is.specific.getImageFilenameHtml());
			StringTools.replaceAll(sb, Constants.TemplateTags.imageCaption, caption);
			StringTools.replaceAll(sb, Constants.TemplateTags.pre, is.pagespecific.getBacklinkDir());
			StringTools.replaceAll(sb, Constants.TemplateTags.back, is.specific.getBacklinkPage());
			StringTools.replaceAll(sb, Constants.TemplateTags.title, "Image: " + caption);
			StringTools.replaceAll(sb, Constants.TemplateTags.meta, xhs.local.meta.toString());
			StringTools.replaceAll(sb, Constants.TemplateTags.width, is.specific.isSmall ? "" : is.specific.getWidthPageAttrib());
			ReplaceTags.replaceWiki2xhtml(sb);
			ReplaceTags.replaceVersion(sb);
			StringTools.replaceAll(sb, Constants.TemplateTags.nextImage, (is.specific.nextLink.length() > 0 ? 
					is.specific.nextLink 
					: is.specific.getBacklinkPage()));
			StringTools.replaceAll(sb, Constants.TemplateTags.prevImage, (is.specific.prevLink.length() > 0 ? 
					is.specific.prevLink 
					: is.specific.getBacklinkPage()));
			StringTools.replaceAll(sb, Constants.TemplateTags.desc, 
					Formattings.format0r(new StringBuffer(is.specific.longdesc.length() > 0 ? is.specific.longdesc : desc), "",
							Formattings.BOLD | Formattings.ITALIC | Formattings.LINKS, false).toString());
	
			/*
			 * Insert the description
			 * 080726 Fixed: &lt;desc&lt; and &gt;desc&gt; not correctly removed
			 */
			int descStart, descEnd;
			
			
			
			if ((descStart = sb.indexOf(">desc>")) >= 0 && (descEnd = sb.indexOf("<desc<")) >= 0) {
				
				/* Remove the block if not needed, otherwise remove position marks */
				if (is.specific.longdesc.length() > 0 || is.specific.desc.length() > 0) {
					sb.delete(descEnd, descEnd + "<desc<".length());
					sb.delete(descStart, descStart + ">desc>".length());
					
				} else { /* No description given, remove everything between &lt;desc&lt; and &gt;desc&gt; */
					sb.delete(descStart, descEnd + "<desc<".length());
				}
				
			}
	
			/*
			 * Remove the navigation part if not necessary
			 */
			int navStart, navEnd;
			while ((navStart = sb.indexOf(">nav>")) >= 0 && (navEnd = sb.indexOf("<nav<")) >= 0) {
				if (is.specific.isGallery) {
					sb.delete(navEnd, navEnd + "<nav<".length());
					sb.delete(navStart, navStart + ">nav>".length());
				} else {
					sb.delete(navStart, navEnd + "<nav<".length());
				}
			}
			
			IOWrite_Stats.writeString(new File(is.specific.getImagepagePath(true, false)), sb.toString(), false);
			
			is.specific.isImagepageCreated = true;
			
		} catch (IOException e) {
			ca.ol("Error: Image page " + is.specific.getImagepagePath(true, false) + " couldn't be created!\n", CommentAtor.ERRORS);
			e.printStackTrace();
			
			is.specific.isImagepageCreated = false;
		} catch (NullPointerException e) { /* GUI mode */ }
		
	}
	
	/**
	 * Links gallery images amongst each other
	 */
	private static class NavLinks {
		
		NavLinks(Specific item) { this.item = item; }
		
		public Specific item;
		
		public boolean isTextOnly() {
			if (item == null || item.text.length() == 0) return false;
			return true;
		}
		
		private NavLinks prevNL = null;
		private NavLinks nextNL = null;
		
		public void setNextNL(NavLinks nextNL) { this.nextNL = nextNL; }
		public void setPrevNL(NavLinks prevNL) { this.prevNL = prevNL; }
		
		/**
		 * @param prev Previous (true) or next (false)
		 * @return Link to previous or next image page.
		 */
		public String getLinkToOther(boolean prev) {
			if ((prev ? prevNL : nextNL) == null)
				return item.getBacklinkPage();
			else
				return (prev ? prevNL : nextNL).getLinkToMyself(prev);
		}
		
		/**
		 * @param prev Direction: towards previous or towards next
		 * @return Path to this or (if current item is only text or a direct link) previous/next image page
		 * @since 3.3.2 direct linking
		 */
		public String getLinkToMyself(boolean prev) {
			if (isTextOnly() || item.direct) {
				if (prev) {
					if (prevNL == null) {
						return item.getBacklinkPage();
					} else {
						return prevNL.getLinkToMyself(prev);
					}
				} else {
					if (nextNL == null) {
						return item.getBacklinkPage();
					} else {
						return nextNL.getLinkToMyself(prev);
					}
				}
			} else {
				return item.getImagepagePath(false, true);
			}
		}
		
	}

}
