package program;


import java.io.PrintStream;

import program.commentator.CommentAtor;
import program.utilities.StringTools;

import regexp.Regex;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Inserts links (external and internal ones).
 * 
 * @author 	Simon A. Eugster
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 * ,		hb9eia
 */
public class WikiLinks {
	
	public static final PrintStream o = System.out;
	
	private static final CommentAtor ca = CommentAtor.getInstance();

	/**
	 * Put all external links between the a-tags. Links of the following form
	 * are recognized: www.server.tld http://server.tld [http://server.tld]
	 * [http://server.tld description] and all with ftp:// too.
	 * 
	 * @param in - input
	 * @return input with links
	 */
	public static StringBuffer makeExternalLinks(StringBuffer in) {
		StringBuffer out = new StringBuffer();
		Regex REX;
		int first, last;
		short counter = 0;

		// /////////////////////////////
		// Make Links from http://anything.tld
		// /////////////////////////////

		/*
		 * Find Links given only as Text, without []. Whitespace has to be in
		 * front of the http; \n is included.
		 */
		REX = new Regex(Resources.Regex.extLinkHTTP, in.toString());

		if (REX.found) {
			last = 0;
			while (REX.find()) {
				counter++;

				// whitespace shall not be in the link!
				// so take the next char (+ 1)
				first = REX.start();
				out.append(in.subSequence(last, first));
				last = REX.end();

				// append the tag <a href=" and the link
				out.append("<a href=\"");
				out.append(in.subSequence(first, last));

				// append the tag "> and use the link as name
				// close tag afterwards
				out.append("\" class=\"external\">");
				out.append(in.subSequence(first, last));
				out.append("</a>");
			}
			// append the rest from the last found pattern
			out.append(in.subSequence(last, in.length()));
			// o.println(in.subSequence(last, in.length()));
		} else
			out = in;

		// /////////////////////////////
		// Make Links from www.anything.tld
		// /////////////////////////////

		/*
		 * Replace Links in the text which are not specially marked with squared
		 * brackets and which start with www. or ftp.
		 */

		in = out;
		out = new StringBuffer();

		REX = new Regex(Resources.Regex.extLinkWWW, in.toString());
		if (REX.found) {
			last = 0;
			while (REX.find()) {
				counter++;

				// whitespace shall not be in the link!
				// so take the next char (+ 1)
				first = REX.start();
				out.append(in.subSequence(last, first));
				last = REX.end();

				// append the opening <a href="http:// tag
				// and the rest of the link starting with www.
				// and close it
				
				out.append("<a href=\"http://");
				
				out.append(in.subSequence(first, last));
				out.append("\" class=\"external\">");

				// use the www-Link as link name and close the tag
				// with the </a>
				out.append(in.subSequence(first, last));
				out.append("</a>");
			}
			out.append(in.subSequence(last, in.length()));
		} else
			out = in;

		// /////////////////////////////
		// Make links from [http://anything.tld Description]
		// /////////////////////////////

		in = out;
		out = new StringBuffer();

		/* The Regular Expression finds [something like this], but [[not that]] */
		REX = new Regex(Resources.Regex.extLinkCommented, in.toString());

		if (REX.found) {
			last = 0;
			int sepPos = 0;
			while (REX.find()) {
				counter++;

				// first char doesn't belong to the link
				first = REX.start();
				out.append(in.subSequence(last, first));
				last = REX.end();
				
				String t = REX.group();

				out.append("<a href=\"");

				if ((sepPos = REX.group().indexOf("|")) <= 0)
						sepPos = t.indexOf(" ");
				
				if (sepPos > 0) {
					/*
					 * Link contains a space or | and thus a description. Therefore
					 * the URL will end at the first position of the separator.
					 */
					out.append(StringTools.replaceAll(new StringBuffer(in.substring(first + 1, first + sepPos)), " ", "%20"));
					out.append("\" class=\"external\">");

					// last is the closing squared bracket -> take last-1
					// at sepPos: space, we don't need this -> +1
					out.append(in.subSequence(first + sepPos + 1, last - 1));
					out.append("</a>");
				} else {
					/*
					 * Link without squared brackets, so take the link itself as
					 * description
					 */
					out.append(in.subSequence(first + 1, last - 1));
					out.append("\" class=\"external\">");
					out.append(in.subSequence(first + 1, last - 1));
					out.append("</a>");
				}
			} // end while
			out.append(in.subSequence(last, in.length()));
		} else { // REX not found
			out = in;
		}

		/* Statistics */
		Statistics.getInstance().counter.linksExt.increase(counter);

		return out;
	}

	/**
	 * Create all internal Links marked with two [[squared brackets]]. URL can
	 * be parted from the name with either a space or a «|».
	 * 
	 * @param in - input
	 * @return linked output
	 */
	public static StringBuffer makeInternalLinks(StringBuffer in,
			String currentFilename) {
		
		StringBuffer funcName = new StringBuffer("Links.makeInternalLinks");
		
		/*
		 * Prevent bugs
		 */
		if (currentFilename == null)
			currentFilename = "";

		StringBuffer out;
		Regex REX = new Regex(Resources.Regex.intLink, in.toString());
		int first, last = 0;
		short counter = 0;
		StringBuffer expression;
		StringBuffer link = new StringBuffer();
		StringBuffer name = new StringBuffer();
		boolean selflink;
		boolean hasName;
		int pos;

		if (REX.found) {
			out = new StringBuffer();
			while (REX.find()) {
				counter++;
				selflink = false;

				first = REX.start();
				out.append(in.subSequence(last, first));
				last = REX.end();
				expression = new StringBuffer(in.subSequence(first + 2,
						last - 2));

				/*
				 * Search for link name and URL.
				 * 
				 * Name from the link: First, look whether link and name are
				 * parted with a «|», and if they are not, look whether they are
				 * parted with a space. Like this, it is possible to avoid
				 * having to escape spaces in links.
				 */
				pos = expression.indexOf("|");
				if (pos > 0) {
					link = new StringBuffer(expression.subSequence(0, pos));
					name = new StringBuffer(expression.subSequence(pos + 1,
							expression.length()));
					hasName = true;

					selflink = currentFilename.equals(link.toString()
							.toLowerCase());

					if (selflink)
						ca.ol(funcName + "Link (current file) with name "
								+ name + ") found.", CommentAtor.DEBUG);
					else
						ca.ol(funcName + "Link (" + link + ") with name ("
								+ name + ") found.", CommentAtor.DEBUG);
				} else {
					pos = expression.indexOf(" ");
					if (pos > 0) {
						link = new StringBuffer(expression.subSequence(0, pos));
						name = new StringBuffer(expression.subSequence(pos + 1,
								expression.length()));
						hasName = true;
						hasName = true;

						selflink = currentFilename.equals(link.toString()
								.toLowerCase());

						if (selflink)
							ca.ol(funcName
									+ "Link (current file) with name " + name
									+ ") found.", CommentAtor.DEBUG);
						else
							ca.ol(funcName + "Link (" + link
									+ ") with name (" + name + ") found.", CommentAtor.DEBUG);
					} else {
						/*
						 * Link has no name
						 */
						link = expression;
						hasName = false;

						selflink = currentFilename.equals(link.toString()
								.toLowerCase());

						if (selflink)
							ca.ol(funcName + "Link (current file) found.",
									CommentAtor.DEBUG);
						else
							ca.ol(funcName + "Link (" + link + ") found.",
									CommentAtor.DEBUG);
					}
				}

				/*
				 * Add link name and URL
				 */
				if (selflink) {
					out.append("<strong class=\"selflink\">");
					if (hasName)
						out.append(name);
					else
						out.append(link);
					out.append("</strong>");
				} else {
					if (link.charAt(0) != '#')
						out.append("<a href=\"./");
					else
						out.append("<a href=\"");
					out.append(link);
					out.append("\" class=\"internal\">");
					if (hasName)
						out.append(name);
					else
						out.append(link);
					out.append("</a>");
				}
			}
			out.append(in.subSequence(last, in.length()));
			
		} else {
			out = in;
		}

		/* Statistics */
		Statistics.getInstance().counter.linksInt.increase(counter);

		return out;
	}

}
