package program;

import java.util.regex.Matcher;

import program.commentator.CommentAtor;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * Handles nowiki-Tags
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class WikiNoWiki {
	
	private static final CommentAtor ca = CommentAtor.getInstance();

	/**
	 * Temporarily removes &lt;nowiki&gt; content.
	 * @param in
	 * @return
	 */
	public static StringBuffer removeNowikiContent(StringBuffer in) {
		Matcher m = Resources.Regex.nowikiContent.matcher(in);
		
		if (!m.find()) {
			return in;
		} else {
			m.reset();
			
			StringBuffer out = new StringBuffer();
			int last = 0;
			while (m.find()) {
				Statistics.getInstance().counter.nowikiRemoved.increase();
				out.append(in.substring(last, m.start()));
				
				XHTML.nowiki.add(m.group(1));
				/*
				 * Insert the nowiki-tags again that the
				 * expression can be inserted afterwards at the right place
				 */
				out.append("<nowiki></nowiki>");
				
				last = m.end();
			}
			out.append(in.substring(last, in.length()));
			
			return out;
		}
	}

	/**
	 * Re-inserts removed &lt;nowiki&gt; content.
	 * @param in
	 * @return
	 */
	public static StringBuffer insertNowikiContent(StringBuffer in) {
		
		Matcher m = Resources.Regex.nowikiContent.matcher(in);
		
		if (!m.find()) {
			return in;
		} else {
			
			m.reset();
			StringBuffer out = new StringBuffer();
			int last = 0;
			int counter = 0;
			
			while (m.find()) {
				
				Statistics.getInstance().counter.nowikiInserted.increase();
				out.append(in.substring(last, m.start()));
				
				/*
				 * Re-insert the expression
				 */
				try {
					out.append(XHTML.nowiki.get(counter));
				} catch (IndexOutOfBoundsException e) {
					ca.ol(String.format("Error while trying to insert nowiki-content Nr. %d!", counter), CommentAtor.ERRORS);
					e.printStackTrace();
				}
				
				counter++;
				last = m.end();
			}
			out.append(in.substring(last, in.length()));
			
			return out;
		}
	}
}
