package program.commentator;

import java.util.ArrayList;

/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * This class handles comments. Supports logging.
 * TODO 5 output log in GUI
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class CommentAtor {

	public static final byte ERRORS = 1;		// 1
	public static final byte MSG = 1 << 1;		// 2
	public static final byte V_MSG = 1 << 2;	// 4
	public static final byte DEBUG = 1 << 3;	// 8
	public static final byte ANY = 1 << 4;		// 16
	
	public byte flags = ERRORS + ANY;
	
	private CommentAtor() {
		flags = ERRORS | MSG| ANY;
	}
	private static CommentAtor cat = new CommentAtor();
	
	public static CommentAtor getInstance() {
		return cat; 
	}
	
	public CommentLog log = new CommentLog();
	
	/**
	 * Prints an object to stdout
	 * @param s
	 * @param flag Importance
	 */
	public final void o(Object s, int flag) {
		if ((flags & flag) != 0) {
			if ((flag & ERRORS) != 0)
				System.err.print(s);
			else
				System.out.print(s);
			log.add(s, flag);
		}
	}
	
	/**
	 * Prints an object line to stdout
	 * @param s
	 * @param flag Importance
	 */
	public final void ol(Object s, int flag) {
		if ((flags & flag) != 0) {
			if ((flag & ERRORS) != 0)
				System.err.println(s);
			else
				System.out.println(s);
			log.add(s, flag);
		}
	}
	
	public void setDebug() { flags |= DEBUG; }
	public void setDead() { flags = 0; }
	public void setSilent() { flags = (byte) (ANY | ERRORS | (flags & DEBUG)); }
	public void setVerbose() { flags |= MSG | V_MSG | DEBUG; }
	
	/**
	 * Comment logger
	 */
	public static class CommentLog {
		
		public CommentLog() { }
		
		private ArrayList<CommentLogItem> log = new ArrayList<CommentLogItem>();
		
		public void add(Object s, int importance) {
			log.add(new CommentLogItem(s, importance));
		}
		
		public StringBuffer getLogByImportance(byte importance) {
			StringBuffer out = new StringBuffer();
			
			for (CommentLogItem cli : log) {
				if ((cli.importance & importance) > 0) {
					out.append(cli.s.toString());
				}
			}
			
			return out;
		}
		
	}
	
	/**
	 * Comment item
	 */
	public static class CommentLogItem {
		
		int importance = 0;
		Object s = null;
		
		public CommentLogItem() { }
		
		public CommentLogItem(Object s, int importance) {
			this.importance = importance;
		}
	}
	
}
