package program.templateHandler;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.regex.Matcher;

import program.Container_Resources;
import program.Resources;
import program.argumentHandler.ArgumentReaderObject;
import program.argumentHandler.ArgumentItem;
import program.commentator.Logger;
import program.utilities.IORead_Stats;

/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * This class will once be very powerful and handle templates of any kind.
 * (Receives the name of a template and its arguments and returns the template text afterwards)
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class Template {
	
	protected File templateSource = null;
	static StringBuffer template = new StringBuffer();
	HashMap<String, ArgumentItem> arguments = new HashMap<String, ArgumentItem>();

	public Template() {	}
	public Template(String templateFilename) {
		templateSource = null;
		File f = new File(templateFilename);
		if (f.exists() && !f.isDirectory() && f.canRead()) {
			templateSource = new File(templateFilename);
		}
	}
	public Template(File templateFile) {
		templateSource = null;
		if (templateFile != null && templateFile.exists() && templateFile.canRead() && !templateFile.isDirectory())
			templateSource = templateFile;
	}
	
	// TODO 0 Template: if... (work from the rear)
	// TODO 0 Template: ifeq
	// TODO 0 Template: ifexist

	// TODO 3 Template: iferror
	// TODO 4 Template: expr
	// TODO 4 Template: ifexpr
	// TODO 4 Template: time 
	// TODO 4 Template: switch
	
	@SuppressWarnings("unused")
	private static int getCorrespondingBrackets(String s) {
		int pos = 0;
		int openBrackets = 0;
		
		for (char c : s.toCharArray()) {
			pos++;
			if (c == '{')
				openBrackets++;
			if (c == '}')
				openBrackets--;
			
			if (openBrackets == 0)
				return pos;
		}
		return -1;
	}
	
	/**
	 * Reads the template file and inserts the parameters.
	 * @param args
	 * @return The complete applied template with inserted parameters
	 */
	public StringBuffer applyTemplate(String args) {
		if (!readTemplate()) {
			template = Container_Resources.readResource(Container_Resources.smissingTemplate);
		}
		
		/** Contains the arguments given when the template was called; 
		 *  the {{{1}}} etc. have to be replaced with them. */
		arguments = program.argumentHandler.ArgumentReader.getArgumentsMap(args);
		/** Contains the arguments of {{{1|alternative}}} tags */
		ArgumentReaderObject aro = new ArgumentReaderObject();
		Matcher m = Resources.Regex.tplParameterInTemplate.matcher(template);
		StringBuffer out = new StringBuffer();
		int last = 0;
		
		/* 
		 * {{{1}}} etc will be replaced if the parameter was set or if
		 * there is an alternative text, otherwise it will be left as is. 
		 */
		while (m.find()) {
			out.append(template.substring(last, m.start()));
			aro.setArguments(m.group(1));
			if (aro.size() > 0) {
				if (arguments.containsKey(aro.argsList.get(0).fullArg)) {
					out.append(arguments.get(aro.argsList.get(0).fullArg).argument);
				} else {
					if (aro.size() > 1) {
						out.append(aro.argsList.get(1).fullArg);
					}
					else {
						out.append(m.group());
					}
				}
			}
			last = m.end();
		}
		out.append(template.substring(last, template.length()));
		
		template = out;
		
		return template;
	}
	
	/**
	 * Reads a template from a file.
	 * @return Success
	 */
	private boolean readTemplate() {
		try {
			template = IORead_Stats.readSBuffer(templateSource);
			return true;
		} catch (IOException e) {
			Logger.getInstance().log.append(e);
			e.printStackTrace();
		}
		return false;
	}
	
}
