package program.templateHandler;

import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Matcher;

import program.Resources;
import program.Statistics;
import program.utilities.IORead;

/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Template applier.
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class TemplateManager {
	
	private static ArrayList<String> cdataSections = new ArrayList<String>();
	
	/**
	 * Applies templates. Templates can be called the following way: <br />
	 * {{:templateName|arg1|arg2|&lt;![CDATA[arg|3]]&gt;}}
	 * @param input
	 * @return
	 */
	public static StringBuffer applyTemplates(StringBuffer input) {
		StringBuffer out = new StringBuffer();
		Template tempTemplate = new Template();
		
		Statistics.getInstance().sw.timeApplyingTemplates.continueTime();
		
		input = removeCdataSections(input);

		Matcher m = Resources.Regex.template.matcher(input);
		int end = 0;
		String tplAllArgs;
		
		while (m.find()) {
			Statistics.getInstance().counter.templates.increase();
			out.append(input.substring(end, m.start()));
			
			if (m.group(1).charAt(0) != '|')
				tplAllArgs = '|' + m.group(1);
			else
				tplAllArgs = m.group(1);
			
			tempTemplate = new Template(TemplateInfo.getTemplateName(tplAllArgs));
			out.append(tempTemplate.applyTemplate(tplAllArgs));
			
			end = m.end();
		}
		out.append(input.substring(end, input.length()));
		
		out = insertCdataSections(out);
		
		Statistics.getInstance().sw.timeApplyingTemplates.stop();
		
		return out;
	}
	
	/**
	 * Removes the content of CDATA sections
	 * @param input
	 * @return input with removed CDATA content
	 */
	private static StringBuffer removeCdataSections(StringBuffer input) {
		StringBuffer out = new StringBuffer();
		cdataSections = new ArrayList<String>();
		Matcher m = Resources.Regex.cdata.matcher(input);
		
		int last = 0;
		while (m.find()) {
			out.append(input.substring(last, m.start()));
			
			cdataSections.add(m.group(1));
			// Replace the CDATA content with the index of the content in the ArrayList cdataSections
			out.append("<![CDATA[" + (cdataSections.size() - 1) + "]]>");
			
			last = m.end();
		}
		out.append(input.substring(last, input.length()));
		
		return out;
	}
	
	/**
	 * Re-inserts remaining CDATA sections with original content 
	 * @param input
	 * @return input with replaced CDATA sections
	 */
	private static StringBuffer insertCdataSections(StringBuffer input) {
		StringBuffer out = new StringBuffer();
		Matcher m = Resources.Regex.cdata.matcher(input);
		
		int last = 0;
		while (m.find()) {
			out.append(input.substring(last, m.start()));
			
			try {
				int pos = Integer.parseInt(m.group(1));
				out.append(cdataSections.get(pos));
			} catch (NumberFormatException e) {
				out.append(m.group());
			}
			
			last = m.end();
		}
		out.append(input.substring(last, input.length()));
		
		return out;
	}
	
	public static StringBuffer testCdataRemover(StringBuffer input) {
		input = removeCdataSections(input);
		input = insertCdataSections(input);
		return input;
	}
	
	public static void main(String[] args) throws IOException {
		StringBuffer sb = IORead.readSBuffer(new java.io.File("test"));
		System.out.println(testCdataRemover(sb));
	}

}
