package program.typo;

import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

import program.utilities.StringTools;

import regexp.Regex;



/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Will perhaps correct some typographical mistakes in future.
 * 
 * TODO 4 Typography correction depending on doc language 
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class Typograph {
	
	private static I18n i18n = I18nFactory.getI18n(Typograph.class, "bin.l10n.Messages", program.Globals.getLocale());
	private StringBuffer content = new StringBuffer();
	
	//Typo: dash spaces(abbr) zoll quot
	
	public void setContent(StringBuffer sb) {
		content = sb;
	}
	
	public StringBuffer getContent() {
		return content;
	}
	
	public void replace(String arguments) {
		content = replace(content, arguments);
	}
	
	public StringBuffer replace(StringBuffer in, String arguments) {
		boolean all = false;
		
		StringBuffer out = in;
		
		if (arguments.indexOf("all") >= 0) 
			all = true;
		
		if (all || arguments.contains("s")) 
			out = makeSpaces(in);
		
		if (all || arguments.contains("d")) 
			out = makeDashes(in);
		
		if (all || arguments.contains("z"))
			out = makeZollChars(in);
		
		if (all || arguments.contains("q"))
			out = makeQuotationMarks(in);
		
		return out;
	}
	
	public StringBuffer makeSpaces(StringBuffer in) {
		StringBuffer out;
		short counter = 0;
		int first, last = 0;

		// \\p{Z}: Unicode-Whitespace
		Regex REX = new Regex("[\\s\\p{Z}(](z\\.B\\.|d\\.h\\.|v\\.a\\.|u\\.a\\.|s\\.o\\.)[\\s)\\p{Z},]", in.toString());
		if (REX.found) {
			out = new StringBuffer();
			while (REX.find()) {
				counter++;
				first = REX.start();
				out.append(in.subSequence(last, first));
				out.append(StringTools.replaceOnce(new StringBuffer(REX.group()), ".", ".&thinsp;"));
				// subtract 1 from the end that the space is added too
				last = REX.end();
			}
			out.append(in.subSequence(last, in.length()));
			
			return out;
		}
		
		return in;
	}
	
	public static void main(String[] args) {
//		Typograph t = new Typograph();
		
//		t.setContent(new StringBuffer("das ist z.B. ein Beispiel,	d.h. es ist wirklich (s.o.) so. text ist u.a.,  "));
//		t.replace("s");
		
//		System.out.println(t.getContent());
	}
	
	public StringBuffer makeDashes(StringBuffer in) {
		StringBuffer out = in;
		short counter = 0;
		int first, last = 0;

		/*
		 * Replace the dashes. Dashes have a' ' before and after the '-'.
		 * Replace the - with &ndash;.
		 */

		Regex REX = new Regex(" - ", in.toString());
		if (REX.found) {
			out = new StringBuffer();
			while (REX.find()) {
				counter++;
				first = REX.start();
				out.append(in.subSequence(last, first + 1));
				out.append("&ndash;");
				// subtract 1 from the end that the space is added too
				last = REX.end() - 1;
			}
			out.append(in.subSequence(last, in.length()));
			counter = 0;
			last = 0;
			in = out;
		}

		REX = new Regex(" -- ", in.toString());
		if (REX.found) {
			out = new StringBuffer();
			while (REX.find()) {
				counter++;
				first = REX.start();
				out.append(in.subSequence(last, first + 1));
				out.append("&ndash;");
				// subtract 1 from the end that the space is added too
				last = REX.end() - 1;
			}
			out.append(in.subSequence(last, in.length()));
			last = 0;
			in = out;
		}
		REX = new Regex(" --,", in.toString());
		if (REX.found) {
			out = new StringBuffer();
			while (REX.find()) {
				counter++;
				first = REX.start();
				out.append(in.subSequence(last, first));
				out.append("&nbsp;&ndash;");
				// subtract 1 from the end that the space is added too
				last = REX.end() - 1;
			}
			out.append(in.subSequence(last, in.length()));
			counter = 0;
			last = 0;
			in = out;
		}
		
		return out;
	}

	public StringBuffer makeZollChars(StringBuffer in) {
		StringBuffer out = new StringBuffer();
		short counter = 0;
		int first, last = 0;

		/*
		 * Replace the inch char
		 */

		// in = out;
		// finds a number followed by \" (" has to be escaped).
		Regex REX = new Regex("\\d+\\\\\" ", in.toString());
		if (REX.found) {
			// last = 0;
			// counter = 0;
			out = new StringBuffer();
			while (REX.find()) {
				counter++;

				first = REX.start();
				out.append(in.subSequence(last, first));
				// don't count space
				last = REX.end() - 1;

				out.append(in.subSequence(first, last - 2));
				out.append(i18n.tr("&Prime;"));
			}
			out.append(in.subSequence(last, in.length()));
			
			return out;
		}
		
		return in;
	}
	
	public StringBuffer makeQuotationMarks(StringBuffer in) {
		StringBuffer out = new StringBuffer();
		short counter = 0;
		int first, last = 0;

		Regex REX = new Regex(" \".+?\"", in.toString());
		if (REX.found) {
			// last = 0;
			out = new StringBuffer();
			while (REX.find()) {
				counter++;

				// don't count space
				first = REX.start() + 1;

				out.append(in.subSequence(last, first));

				// add specific quotation marks
				out.append(i18n.tr("&ldquo;"));

				last = REX.end();
				out.append(in.subSequence(first + 1, last - 1));

				out.append(i18n.tr("&rdquo;"));
			}
			out.append(in.subSequence(last, in.length()));

//			notes.append(i18n.tr("Note: if you want to make a zoll char instead of a quotation, put a backslash (\\) before the char: \\\"\n"));
			
			return out;
		}
		
		return in;
	}

}
