package regexp;

import java.util.regex.*;


/*
 *   LICENSE
 * 
 *   wiki2xhtml is thought to be an effective and efficient way to create
 *   complete webpages from wikitext files.
 *   Copyright (C) 2008  Simon A. Eugster
 *                       Ob. Waldhofstr. 14
 *                       CH-9240 Uzwil

 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.

 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * A class for working with regular expressions (and being quite useless?).
 * TODO 5 un-use it.
 * 
 * @author 	Simon A. Eugster, hb9eia
 * ,		CH-9240 Uzwil
 * ,		simon.eu@gmail.com
 */
public class Regex {

	/*
	 * Regular Expressions: Characters \r - Carriage return (Window$: Enter = CR +
	 * LF; Unix: LF) \n - Line feed
	 */
	public String REGEX = new String(), INPUT = new String();

	public Pattern pattern;

	public Matcher matcher;

	public boolean found = false;

	public Regex() {
		;
	}
	public Regex(String regex, String input) { 
		setup(regex, input, 0);
	}

	/**
	 * Find regular expressions. <br>
	 * Escape character is \, but has to have a \ before (intern) compiling;
	 * looks like this: <br>
	 * "\\[" <i>//search for [ </i><br>
	 * "\\\" <i>//search for \ </i><br>
	 * "\\d" <i>//search for a digit</i>
	 * 
	 * @param regex - regular expression
	 * @param input - text to find expressions in
	 * @param flags - Flags like Pattern.MULTILINE
	 */
	public Regex(String regex, String input, int flags) {
		setup(regex, input, flags);
	}
	
	public Regex(Pattern pat, String input) {
		setup(pat, input, 0);
	}
	
	private void setup(String regex, String input, int flags) {
		REGEX = regex; // Regular Expression.
		INPUT = input;

		/*
		 * pattern = Pattern.compile(REGEX, Pattern.MULTILINE); would enable
		 * Multilines; ^ for line begin and $ for line end are possible (not
		 * only file begin or file end)
		 */

		// make "\\" to "\" etc
		// pattern = Pattern.compile(REGEX, Pattern.MULTILINE);
		// can be selected in the pattern itself
		// with a (?m) first
		pattern = Pattern.compile(REGEX);
		matcher = pattern.matcher(INPUT);

		found = matcher.find();
		matcher.reset();
	}
	
	private void setup(Pattern regex, String input, int flags) {
		REGEX = regex.toString(); // Regular Expression.
		INPUT = input;

		pattern = regex;
		matcher = pattern.matcher(INPUT);

		found = matcher.find();
		matcher.reset();
	}

	/**
	 * Find the regular expression in INPUT or go to next one.<br>
	 * start(), end(), group() are updated.
	 * 
	 * @return true, if expression was found.
	 */
	public boolean find() {
		if (pattern == null) {
			notDefined("pattern");
			return false;
		}
		
		return matcher.find();
	}

	/**
	 * resets the position of the "finder"; search is started from the
	 * beginning.
	 */
	public void reset() {
		matcher.reset();
	}

	/**
	 * @return start position of found expression
	 */
	public int start() {
		return matcher.start();
	}

	/**
	 * @return end position of found expression
	 */
	public int end() {
		return matcher.end();
	}

	/**
	 * @return found expression
	 */
	public String group() {
		return matcher.group();
	}
	
	/**
	 * @param which
	 * @return The which-th captured group
	 */
	public String group(int which) {
		return matcher.group(which);
	}

	/**
	 * Output error message
	 * 
	 * @param what -
	 *            say what is not defined
	 */
	private void notDefined(String what) {
		System.err.println(what + " is not defined");
	}

	/**
	 * Changes the INPUT string (where the Regular Expression shall be searched)
	 * 
	 * @param s -
	 *            new INPUT string
	 */
	public void setInput(String s) {
		if (s.equals(INPUT))
			return;
		INPUT = s;
		refreshMatcher();
	}

	/**
	 * "refreshes" the matcher, if for example INPUT has changed.
	 */
	public void refreshMatcher() {
		matcher = pattern.matcher(INPUT);

	}

	/**
	 * Prints a pattern.
	 * 
	 * @param pat -
	 *            pattern to print
	 */
	public static void printPattern(Pattern pat) {
		System.out.println("Regular Expression: " + pat);

	}
	
	/**
	 * 
	 * @param regex
	 * @param in
	 * @param replace
	 * @return null, if regex or in is empty/null of replace is null, otherwise the input with replaced text. 
	 */
	public static StringBuffer replaceRegexp(String regex, String in, String replace) {
		if (in == null || in.length() == 0 || regex.length() == 0 || regex == null || replace == null)
			return null;
		
		Regex regexp = new Regex(regex, in); 
		StringBuffer out = new StringBuffer();
		
		int first, last = 0;
		while (regexp.find()) {
			first = regexp.start();

			out.append(in.subSequence(last, first));
			out.append(replace);

			last = regexp.end();
		}
		out.append(in.subSequence(last, in.length()));
		return out;
	}

}
